Package: minimaxApprox
Type: Package
Title: Implementation of Remez Algorithm for Polynomial and Rational
        Function Approximation
Version: 0.5.0
Date: 2026-01-08
Authors@R: person(given="Avraham", family="Adler",role=c("aut", "cre", "cph"),
    email="Avraham.Adler@gmail.com", comment = c(ORCID = "0000-0002-3039-0703"))
Description: Implements the algorithm of Remez (1962) for polynomial minimax
    approximation and of Cody et al. (1968) <doi:10.1007/BF02162506> for
    rational minimax approximation.
License: MPL-2.0
URL: https://github.com/aadler/MiniMaxApprox
BugReports: https://github.com/aadler/MiniMaxApprox/issues
Imports: stats, graphics
Suggests: tinytest, covr
ByteCompile: yes
NeedsCompilation: yes
Encoding: UTF-8
UseLTO: yes
Packaged: 2026-01-08 16:58:13 UTC; Parents
Author: Avraham Adler [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3039-0703>)
Maintainer: Avraham Adler <Avraham.Adler@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-08 21:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 20:24:44 UTC; unix
Archs: minimaxApprox.so.dSYM
