useDynLib(minimaxApprox, .registration = TRUE)

# Imports
importFrom(stats,
           optimize,
           uniroot)
importFrom(graphics,
           abline,
           legend,
           points,
           title)

# Exports
export(minimaxApprox,
       minimaxEval,
       minimaxErr,
       print.minimaxApprox,
       plot.minimaxApprox,
       coef.minimaxApprox)

# Methods
S3method(print, minimaxApprox)
S3method(plot, minimaxApprox)
S3method(coef, minimaxApprox)
